@@ -24,18 +24,29 @@ class Plot:
     def __init__(self, title='plot'):
         self.title = title
         self.points = []
+        self.lines = []
+        self.texts = []
         self.x_min =  math.inf
         self.x_max = -math.inf
         self.y_min =  math.inf
         self.y_max = -math.inf
 
-    def point(self, x, y, r=0, g=0, b=0, a=1):
+    def point(self, x, y, r=0, g=0, b=0, a=255):
         y = -y
         self.points.append([x, y, r, g, b, a])
-        self.x_min = min(x, self.x_min)
-        self.x_max = max(x, self.x_max)
-        self.y_min = min(y, self.y_min)
-        self.y_max = max(y, self.y_max)
+        self._include(x, y)
+
+    def line(self, xi, yi, xf, yf, r=0, g=0, b=0, a=255):
+        yi = -yi
+        yf = -yf
+        self.lines.append([xi, yi, xf, yf, r, g, b, a])
+        self._include(xi, yi)
+        self._include(xf, yf)
+
+    def text(self, s, x, y, r=0, g=0, b=0, a=255):
+        y = -y
+        self.texts.append([s, x, y, r, g, b, a])
+        self._include(x, y)
 
     def show(self, w=640, h=480):
         media.init(w, h, title=self.title)
@@ -132,16 +143,21 @@ class Plot:
                     if key == 'Return': media.capture_start()
             # draw
             media.clear(color=(0, 0, 0))
-            self.vertex_buffer.draw()
+            for i in self.vertex_buffers: i.draw()
             margin_x = 2.0 / media.width()  * view.w
             margin_y = 2.0 / media.height() * view.h
             aspect = media.height() / media.width() * view.w / view.h
+            text_h = 10.0/media.height()*view.h
+            ## texts
+            for (s, x, y, r, g, b, a) in self.texts:
+                media.vector_text(s, x=x, y=y-text_h/4, h=text_h, r=r, g=g, b=b, a=a)
+                media.line(x=x, y=y, w=text_h, h=0, r=r, g=g, b=b, a=a)
             ## x axis
             x_divs = media.width() // 200
             i = view.x + view.w / x_divs
             while i < view.x + (x_divs*2-1) * view.w / (x_divs*2):
                 s = '{:.8}'.format(i)
-                media.vector_text(s, x=i+margin_x, y=view.y+view.h-margin_y, h=10.0/media.height()*view.h, aspect=aspect)
+                media.vector_text(s, x=i+margin_x, y=view.y+view.h-margin_y, h=text_h, aspect=aspect)
                 media.line(xi=i, xf=i, y=view.y+view.h, h=-12.0/media.height()*view.h)
                 i += view.w / x_divs
             ## y axis
@@ -149,7 +165,7 @@ class Plot:
             i = view.y + view.h / y_divs
             while i < view.y + (y_divs*2-1) * view.h / (y_divs*2):
                 s = '{:.8}'.format(-i)
-                media.vector_text(s, x=view.x+margin_x, y=i-margin_y, h=10.0/media.height()*view.h, aspect=aspect)
+                media.vector_text(s, x=view.x+margin_x, y=i-margin_y, h=text_h, aspect=aspect)
                 media.line(x=view.x, w=12.0/media.width()*view.w, yi=i, yf=i)
                 i += view.h / y_divs
             ## display
@@ -157,9 +173,24 @@ class Plot:
             media.capture_finish('plot.png')
 
     def _construct(self):
-        self.vertex_buffer = media.VertexBuffer(len(self.points))
+        # points
+        points = media.VertexBuffer(len(self.points))
         for i, point in enumerate(self.points):
-            self.vertex_buffer.update(i, *point)
+            points.update(i, *point)
+        # lines
+        lines = media.VertexBuffer(2*len(self.lines))
+        lines.set_type('lines')
+        for i, (xi, yi, xf, yf, r, g, b, a) in enumerate(self.lines):
+            lines.update(2*i+0, xi, yi, r, g, b, a)
+            lines.update(2*i+1, xf, yf, r, g, b, a)
+        # order
+        self.vertex_buffers = [lines, points]
+
+    def _include(self, x, y):
+        self.x_min = min(x, self.x_min)
+        self.x_max = max(x, self.x_max)
+        self.y_min = min(y, self.y_min)
+        self.y_max = max(y, self.y_max)
 
 def _default_transform(x, y, series=0):
     colors = [
@@ -206,6 +237,19 @@ def plot_scatter_ys(x, ys, transform=_default_transform, title='plot'):
             plot.point(*transform(x[i], y[i], j))
     plot.show()
 
+def plot_dict(d, transform=_default_transform, title='plot'):
+    plot = Plot(title)
+    for x, y in d.items():
+        plot.point(*transform(x, y))
+    plot.show()
+
+def plot_dicts(ds, transform=_default_transform, title='plot'):
+    plot = Plot(title)
+    for j, d in enumerate(ds):
+        for x, y in d.items():
+            plot.point(*transform(x, y, j))
+    plot.show()
+
 def _type_r(v, max_depth=None, _depth=0):
     if type(v) in [int, float]: return 'number'
     if max_depth != None and _depth == max_depth:
@@ -226,6 +270,8 @@ def plot(*args, transform=_default_transform, title='plot'):
     if len(args) == 1:
         if   _is_dim(args[0], 1): plot_func = plot_list
         elif _is_dim(args[0], 2): plot_func = plot_lists
+        elif type(args[0]) == dict: plot_func = plot_dict
+        elif _type_r(args[0]) == _type_r([{}]): plot_func = plot_dicts
     elif len(args) == 2:
         if   _is_dim(args[0], 1) and _is_dim(args[1], 1): plot_func = plot_scatter
         elif _is_dim(args[0], 2) and _is_dim(args[1], 1): plot_func = plot_scatter_xs
